package com.kaixed.kchat.ui.activity

import android.os.Bundle
import android.view.View
import android.widget.EdgeEffect
import androidx.activity.enableEdgeToEdge
import androidx.appcompat.app.AppCompatActivity
import androidx.core.widget.addTextChangedListener
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.recyclerview.widget.RecyclerView.EdgeEffectFactory
import com.kaixed.kchat.databinding.ActivitySearchBinding
import com.kaixed.kchat.model.search.SearchItem
import com.kaixed.kchat.ui.adapter.SearchResultAdapter

class SearchActivity : AppCompatActivity() {
    private var binding: ActivitySearchBinding? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        this.enableEdgeToEdge()
        binding = ActivitySearchBinding.inflate(layoutInflater)
        setContentView(binding!!.root)

        setOnClickListener()

        setupRecycleView()

        binding!!.etSearch.addTextChangedListener(
            afterTextChanged = { s ->
                s?.let {
                    setupViewVisibility(s.length)
                }
            }
        )
    }

    private fun setupViewVisibility(length: Int) {
        binding!!.rvSearchResult.visibility = if (length > 0) View.VISIBLE else View.INVISIBLE
        binding!!.tvPageSetting.visibility = if (length > 0) View.INVISIBLE else View.VISIBLE
    }

    private fun setupRecycleView() {
        binding!!.rvSearchResult.edgeEffectFactory = object : EdgeEffectFactory() {
            override fun createEdgeEffect(view: RecyclerView, direction: Int): EdgeEffect {
                return object : EdgeEffect(view.context) {
                    override fun onPull(deltaDistance: Float, displacement: Float) {
                        // 禁止下拉时的效果
                    }

                    override fun onRelease() {
                        // 禁止释放时的效果
                    }

                    override fun onAbsorb(velocity: Int) {
                        // 禁止吸收时的效果
                    }
                }
            }
        }

        val searchResultAdapter: SearchResultAdapter = getAdapter()
        binding!!.rvSearchResult.adapter = searchResultAdapter
        val layoutManager = LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false)
        binding!!.rvSearchResult.layoutManager = layoutManager
    }

    private fun setOnClickListener() {
        binding!!.tvCancel.setOnClickListener { v ->
            finish()
        }
    }

    private fun getAdapter(): SearchResultAdapter {
        val contactItem = SearchItem("sa", "kaixed", "as", false, "联系人")
        val chatHistoryItem = SearchItem("sa", "kaixed", "as", false, "群聊")
        val groupItem = SearchItem("sa", "kaixed", "as", false, "聊天记录")

        val searchItem1 = SearchItem("sa", "kaixed", "as", true, "联系人")
        val searchItem2 = SearchItem("sa", "kaixed", "as", true, "群聊")
        val searchItem3 = SearchItem("sa", "kaixed", "as", true, "聊天记录")


        val objects: MutableList<Any> = ArrayList(12)
        objects.add("联系人")
        objects.add(contactItem)
        objects.add(contactItem)
        objects.add(searchItem1)

        objects.add("群聊")
        objects.add(groupItem)
        objects.add(groupItem)
        objects.add(searchItem2)

        objects.add("聊天记录")
        objects.add(chatHistoryItem)
        objects.add(chatHistoryItem)
        objects.add(searchItem3)

        val searchResultAdapter = SearchResultAdapter(objects)
        return searchResultAdapter
    }
}
